#ifndef FUNKCE_H_
#define FUNKCE_H_

#include "stm_core.h"
#include <nucleo_usart.h>

// definice konstant pro rizeni sbernice a resetu
#define BC1_SET 1 << 8
#define BC1_RESET 1 << 24

#define BDIR_SET 1 << 9
#define BDIR_RESET 1 << 25

#define DA0_DA7_RESET 0x00FF0000
#define DA0_DA7_SET 0xFF

#define PC_13_SET 1 << 13
#define PC_13_RESET 1 << 29

// vyctove typy enum pro ovladani kanalu, registru a obalek
typedef enum
{
  CHANNEL_A_TONE_FINE,
  CHANNEL_A_TONE_COARSE,

  CHANNEL_B_TONE_FINE,
  CHANNEL_B_TONE_COARSE,

  CHANNEL_C_TONE_FINE,
  CHANNEL_C_TONE_COARSE,

  NOISE_PERIOD,

  ENABLE_NOISE_TONE,

  CHANNEL_A_AMPLITUDE,
  CHANNEL_B_AMPLITUDE,
  CHANNEL_C_AMPLITUDE,

  ENVELOPE_PERIOD_FINE,
  ENVELOPE_PERIOD_COARSE,
  ENVELOPE_SHAPE_CYCLE,
} registry;

typedef enum
{
  CHANNEL_TONE_NONE,
  CHANNEL_TONE_A,
  CHANNEL_TONE_B,
  CHANNEL_TONE_C,
  CHANNEL_TONE_AB,
  CHANNEL_TONE_AC,
  CHANNEL_TONE_BC,
  CHANNEL_TONE_ABC,
} kanaly_ton;

typedef enum
{
  CHANNEL_NOISE_NONE,
  CHANNEL_NOISE_A,
  CHANNEL_NOISE_B,
  CHANNEL_NOISE_C,
  CHANNEL_NOISE_AB,
  CHANNEL_NOISE_AC,
  CHANNEL_NOISE_BC,
  CHANNEL_NOISE_ABC,
} kanaly_noise;

typedef enum
{
  NOISE_OFF,
  NOISE_ON,
} noise;

typedef enum
{
  ENVELOPE_OFF,
  ENVELOPE_ON,
} envelope;

typedef enum
{
  rampa_z_1_do_0 = 1,
  trojuhelnikovy_pulz_z_0_do_0,
  trojuhelnik_z_1_do_0,
  pila_z_1,
  trojuhelnikovy_pulz_z_1_do_1,
  trojuhelnik_z_0_do_1,
  rampa_z_0_do_1,
  pila_z_0,
} obalky;

void bus_init(void);                           // nastaveni sbernice - DA0-DA7, BDIR, BC2
void reset_init(void);                         // nastaveni reset tlacitka na pinu PC_13
bool read_reset_btn(void);                     // cteni hodnoty reset tlacika
void clock_init(void);                         // nastaveni citace pro generovani 1 MHz hodinoveho signalu
void bus_inactive(void);                       // nastaveni sbernice do neaktivniho stavu
void bus_write(void);                          // priprav sbernici pro zapis hodnoty do registru
void bus_latch_adress(void);                   // priprav sbernici pro vyber registru
void bus_reset(void);                          // nastavi DA0 -DA7 na nulu
void zapis_do_registru(uint8_t registr, uint8_t hodnota); // vyber a zapis hodnoty do registru

// vyber kanalu na kterem se bude prehravat a povoleni generatoru sumu
void vyber_kanalu_tonu_noise(kanaly_ton channel, noise povolit_noise);

// nastaveni frekvence tonu na vybranem kanalu
void nastaveni_frekvence_tonu(kanaly_ton channel, uint32_t frekvence);

// nastaveni urovne D/A prevodniku na danem kanalu a povoleni obalkoveho generatoru
void nastaveni_amplitudy_kanalu(kanaly_ton channel, uint8_t amplituda, envelope povolit_envelope);
void nastaveni_frekvence_noise_generatoru(uint32_t frekvence);              // nastaveni frekvence generatoru sumu
void nastaveni_frekvence_tvaru_obalky(float frekvence, obalky tvar_obalky); // nastaveni frekvence a tvaru obalky
void delay_ms(uint32_t cas);                  // generovani casoveho zpozdeni
void SysTick_Handler(void);                   // funkce pro obsluhu preruseni od casovace SysTick

#endif /* FUNKCE_H_ */
