#include <mpp_shield.h>
#include <funkce.h>

// nastaveni rezimu ovladani AY-3-8912
//#define mod_reset   // tlacitko na pinu PC13 plni funkci RESETu pro AY-3-8912, nepouzivat pro mod_skladba
#define mod_skladba // tlacitko na pinu PC13 plni funkci prepinani skladeb, nepouzivat pri mod_reset
//#define ovladani_vystupu  // ovladani jednotlivych funkci AY-3-8912 jako je nastaveni obalky, frekvence, kanalu
                          // nepouzivat s mod_skladba, pouzivat mod_reset

// pristup ke zvukovym souboru pri pouziti mod_skladba
#ifdef mod_skladba
#include <foty2.h>
#include <terminator.h>
#include <batman.h>
#include <garylineker.h>
#include <f15_strike_eagle.h>
#endif

int main(void)
{
  bool min_stav_reset_btn =  true;

// promenne pro mod_reset
#ifdef mod_reset
  bool zmacknuto = false;
#endif

// promenne pro mod_skladba
#ifdef mod_skladba
  uint8_t cislo_registru = 0;
  uint8_t hodnota_do_registru = 0;
  uint8_t cislo_skladby = 0;
  uint32_t posuv_reg = 5;
  uint32_t posuv_hodnota = 6;
  uint8_t *pole = NULL;
#endif

  Usart2Init(9600);   // nastaveni rychlosti USARTu na 9600 Baud
  puts("\n---------------------------\n");
  puts("Start aplikace\n");

  SystemCoreClockUpdate();  // aktualizuje globalni promennou SystemCoreClock
  SysTick_Config(SystemCoreClock / 1000); // citac SysTick pretece za 1ms, frekvence citace je 1 kHz

// inicializace hodin, sbernice a pocatecni RESET AY-3-8912
  clock_init();
  bus_init();
  bus_inactive();
  bus_reset();
  reset_init();

// funkce pro ovladani mod_skladba
#ifdef   ovladani_vystupu
  vyber_kanalu_tonu_noise(CHANNEL_TONE_B, NOISE_OFF); // kanal B, sum vypnuty
  nastaveni_frekvence_tonu(CHANNEL_TONE_B, 1000);     // kanal B, frekvence 1000 Hz
  nastaveni_amplitudy_kanalu(CHANNEL_TONE_B, 15, ENVELOPE_OFF); // kanal B, amplituda 15, obalka je vypnuta
  nastaveni_frekvence_tvaru_obalky(0.15, pila_z_1);   // frekvence obalky a jeji tvar
  nastaveni_frekvence_noise_generatoru(3000);       // frekvence sumoveho generatoru je 3000 Hz
#endif

//  vypnuti kanalu pro mod_skladba, kdyz nebude zvolena skladba tak vystup nebude aktivni
#ifdef mod_skladba
  vyber_kanalu_tonu_noise(CHANNEL_TONE_NONE, CHANNEL_TONE_NONE);  // pocatecni nastaveni
#endif

  while (1)
  {
#ifdef mod_reset
    // cteni stavu tlacitka RESET, kdyz bylo stisknuto tak vysledek bude false
    bool soucasny_stav_reset_btn = read_reset_btn();

    if(min_stav_reset_btn != soucasny_stav_reset_btn) // porovnani soucasneho a predchoziho stavu
    {
      min_stav_reset_btn = soucasny_stav_reset_btn;   // ulozeni soucasneho stavu

      if(!(soucasny_stav_reset_btn && zmacknuto))   // detekce stisku tlacitka, podminka funguje pro prvni pruchod funkci
      {
        zmacknuto = true;             // stisk byl detekovan
        bus_inactive();                // nastaveni sbernice do neaktivniho stavu
        bus_reset();                  // nastavi DA0 -DA7 na nulu
        GPIOC->BSRR |= PC_13_RESET;   // nastaveni reset casti do 1 -> PC13 = 0 -> reset AY-8912 (je aktivni na log. 0)
      }

      if(soucasny_stav_reset_btn && zmacknuto) // podminka funguje pro druhy pruchod funkci (po stisku tlacitka)
      {
        zmacknuto = false;            // nastaveni na false pro dalsi detekci stisku
        GPIOC->BSRR |= PC_13_SET;     // nastaveni set casti do 1 -> PC13 = 1 (set ma vyssi prioritu ne reset)
      }
    }
#endif

#ifdef mod_skladba
    // cteni stavu tlacitka pro vyber skladby, kdyz bylo stisknuto tak vysledek bude false
    bool soucasny_stav_reset_btn = read_reset_btn();

    if(min_stav_reset_btn != soucasny_stav_reset_btn) // porovnani soucasneho a predchoziho stavu
    {
      min_stav_reset_btn = soucasny_stav_reset_btn; // ulozeni soucasneho stavu

      if(soucasny_stav_reset_btn) // detekce stisku tlacitka, podminka funguje pri druhem pruchodu
      {
        cislo_skladby++;        // zvoleni skladby

        // pro nastaveni pocatecniho stavu na nulu nebo kdyz chceme behem aktualniho prehravani skladby prehrat jinou
        cislo_registru = 0;
        hodnota_do_registru = 0;
        posuv_reg = 5;
        posuv_hodnota = 6;

        if(cislo_skladby == 6)  //  sesty stisk tlacitka znamena konec prehravani
        {
          cislo_skladby = 0;
          vyber_kanalu_tonu_noise(CHANNEL_TONE_NONE, CHANNEL_TONE_NONE);
          pole = NULL;
        }
      }
    }

    if (cislo_skladby > 0)  // probehne kdyz je zvolena skladba
    {
      switch(cislo_skladby) // vyber skladby podle poctu stisku tlacitka
      {
        case 1:
          pole = FOTY2_music_array;  // prirazeni adresy hudebniho souboru do promenne pole
          break;
        case 2:
          pole = F15_STRIKE_EAGLE_music_array;
          break;
        case 3:
          pole = GARYLINEKER_music_array;
          break;
        case 4:
          pole = TERMINATOR_music_array;
          break;
        case 5:
          pole = BATMAN_music_array;
          break;
      }
    }

    if(pole != NULL)  // probehne kdyz adresa pole nebude NULL (neni zvolena skladba)
    {
      cislo_registru = pole[posuv_reg]; // prochazeni hudebniho souboru, vyber registru
      hodnota_do_registru = pole[posuv_hodnota];  // prochazeni hudebniho souboru, vyber hodnoty
      posuv_reg += 2;     // pricteni pro skok na dalsi adresu registru
      posuv_hodnota += 2; // pricteni pro skok na dalsi hodnotu registru

      if(cislo_registru < 16) // detekce zapisovych dat
      {
        zapis_do_registru(cislo_registru, hodnota_do_registru); // zapis hodnot ze souboru na cip
      }

      if(cislo_registru == 16)  // detekce zpozdeni
      {
        delay_ms(20 * (hodnota_do_registru + 1)); // zpozdeni pred zpracovani dalsiho datoveho bloku
      }

      // detekce konce souboru a nastaveni pocatecnich hodnot, aby mohla zvolena skladba hrat od zacatku
      if(cislo_registru == 0xff)
      {
        cislo_registru = 0;
        hodnota_do_registru = 0;
        posuv_reg = 5;
        posuv_hodnota = 6;
      }
    }
#endif
  }
}

